/*
 * This file is part of the libpayload project.
 *
 * Copyright (C) 2008 Advanced Micro Devices, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <libpayload.h>

/*
 * This structure seeds the stack. We provide the return address of our main
 * function, and further down, the address of the function that we call when
 * we leave and try to restore the original stack. At the very bottom of the
 * stack we store the original stack pointer from the calling application.
 */

static void start_main(void);
extern void _leave(void);

static struct {
	uint32_t eip[2];
	uint32_t raddr[2];
	uint32_t esp;
} initial_stack __attribute__ ((section(".istack"))) = {
	{ (uint32_t) start_main, 0 },
	{ (uint32_t) _leave, 0 },
	(uint32_t) & initial_stack,
};

void *_istack = &initial_stack;

/**
 * This is our C entry function - set up the system
 * and jump into the payload entry point.
 */
static void start_main(void)
{
	extern int main(void);

	/* Set up the consoles. */
	console_init();

	/* Gather system information. */
	lib_get_sysinfo();

	/*
	 * Any other system init that has to happen before the
	 * user gets control goes here.
	 */

	/* Go to the entry point. */

	/* In the future we may care about the return value. */
	(void) main();

	/*
	 * Returning here will go to the _leave function to return
	 * us to the original context.
	 */
}
